﻿using System;
using System.Collections.Generic;
using Interfaces;
using Interfaces.Services;

namespace Services
{

	public class CommandDispatcherService<T> : ICommandDispatcherService<T>
	{
		/// <summary>
		/// List of commands that can be executed.
		/// </summary>
		internal Dictionary<T, IDispatchedCommand<T>> _executableCommands;

		/// <summary>
		/// The command to execute if a given key is not found.  If this is null,
		/// an ArgumentException will be thrown.
		/// </summary>
		public IDispatchedCommand<T> KeyNotFoundCommand{ get; set; }

		public CommandDispatcherService() 
		{
			_executableCommands = new Dictionary<T, IDispatchedCommand<T>>();
		}

		/// <summary>
		/// Executes the command associated with the given key.
		/// </summary>
		public void ExecuteCommand(T pCommandKey, object pParameter)
		{
			if (pCommandKey == null) throw new ArgumentNullException("CommandKey");

			var command = GetCommand(pCommandKey);
			command.Execute(pParameter);
			command.ExecuteCount++;
		}

		private IDispatchedCommand<T> GetCommand(T pCommandKey)
		{
			if (_executableCommands.ContainsKey(pCommandKey)) 
				return _executableCommands[pCommandKey];
			if (KeyNotFoundCommand != null) return KeyNotFoundCommand;
			throw new ArgumentOutOfRangeException(pCommandKey.ToString());
		}

		/// <summary>
		/// Adds a given command to the list of available commands
		/// </summary>
		public void AddCommand(IDispatchedCommand<T> pCommand)
		{
			if (pCommand == null) throw new ArgumentNullException("Command");
			_executableCommands.Add(pCommand.CommandKey, pCommand);
		}

		/// <summary>
		/// Clears out all commands from list.
		/// </summary>
		public void ClearCommands()
		{
			_executableCommands.Clear();
		}
	}
}
