﻿using System;
using System.Windows.Forms;
using Infrastructure;
using Interfaces.Services;
using Interfaces.Views;

namespace Views_Winforms
{
	public class BaseView : UserControl, IBaseView
	{
		/// <summary>
		/// Parent region this control is hosted by
		/// </summary>
		internal Panel _parentRegion;

		/// <summary>
		/// Places this view into the given region.  It 
		/// does not activate the view - this must be done
		/// by the caller.
		/// </summary>
		public void SetRegion(string pRegion)
		{
			var regionMgr = ServiceLocator.Resolve<IWinformsRegionManager>();
			var newParentRegion = regionMgr.Get(pRegion);
			if (newParentRegion == null)
			{
				//we are not prepared for regions yet, so exit
				//when the conversion to a RegionManager is done,
				//this should throw an exception
				return;
			}

			//remove from current owner region, if necessary
			//this can happen if we decide to move a view at runtime
			//This is not a good idea, but plan for it anyway
			if (_parentRegion != null && _parentRegion.Controls.Contains(this))
				_parentRegion.Controls.Remove(this);

			//save off our new parent
			_parentRegion = newParentRegion;

			//we do not want the view to display yet.
			//This is so we can eagerly load the view and not
			//display it until it is necessary.  See the 
			//Activate method for making the view visible
			this.Visible = false;

			//set new region as our parent
			_parentRegion.Controls.Add(this);
			Dock = DockStyle.Fill;
		}

		/// <summary>
		/// makes this view visible and brings it to front.
		/// This should be called anytime focus switches from
		/// a different view to this one.
		/// </summary>
		public virtual void SetActive()
		{
			//make sure we have been assigned a region first
			if (_parentRegion == null)
				throw new InvalidOperationException(
					"Please set host region before activating");

			//bring the region to the front first
			_parentRegion.BringToFront();

			//now make this view visible and bring to front
			this.Visible = true;
			this.BringToFront();
		}

		/// <summary>
		/// This shoud be called anytime focus switches away from this view
		/// to something else.
		/// </summary>
		public virtual void SetInActivate()
		{
			//this is here so the descendants can override
			//it in case they need to do anything when they
			//lose focus
		}
	}
}
